"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
class UpdatePluginCommand {
    constructor($pluginsService, $errors) {
        this.$pluginsService = $pluginsService;
        this.$errors = $errors;
        this.allowedParameters = [];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let pluginNames = args;
            if (!pluginNames || args.length === 0) {
                let installedPlugins = yield this.$pluginsService.getAllInstalledPlugins();
                pluginNames = installedPlugins.map(p => p.name);
            }
            for (let p of pluginNames) {
                yield this.$pluginsService.remove(p);
                yield this.$pluginsService.add(p);
            }
        });
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!args || args.length === 0) {
                return true;
            }
            let installedPlugins = yield this.$pluginsService.getAllInstalledPlugins();
            let installedPluginNames = installedPlugins.map(pl => pl.name);
            let pluginName = args[0].toLowerCase();
            if (!_.some(installedPluginNames, name => name.toLowerCase() === pluginName)) {
                this.$errors.failWithoutHelp(`Plugin "${pluginName}" is not installed.`);
            }
            return true;
        });
    }
}
exports.UpdatePluginCommand = UpdatePluginCommand;
$injector.registerCommand("plugin|update", UpdatePluginCommand);
